/************************************************************

    ViewTypes.r
	MacApp 'view' type template definitions


    Copyright Apple Computer, Inc. 1985-1990
    All rights reserved.

************************************************************/

#ifndef __ViewTypes__
#define __ViewTypes__

/*------------------------------------Equates-----------------------------------*/

/*	The following are used to set adornments.  Simply add for desired value.	*/
#define adnNone 		0b00000000
#define adnLineTop		0b00000001
#define adnLineLeft 	0b00000010
#define adnLineBottom	0b00000100
#define adnLineRight	0b00001000
#define adnFrame		0b00001111
#define adnPatFill		0b00010000
#define adnOval 		0b00100000
#define adnRRect		0b01000000
#define adnShadow		0b10000000

#define noAdornment 	adnNone, {1, 1}
#define arrowsAndBackspace	0xF0000100
#define carriageReturn	0x00002000
#define black			{0, 0, 0}
#define noInset 		{0, 0, 0, 0}
#define stdInset 		{3, 3, 3, 3}

#define systemFont		plain,	0, {0, 0, 0}, ""
#define applFont9		plain,	9, {0, 0, 0}, "a"
#define applFont12		plain, 12, {0, 0, 0}, "a"

#define IncludeViews	noID, {0, 0}, {0, 0}, 0, 0, 0, 0, Include
#define IncludeViewsAt	noID, {0, 0}, {0, 0}, 0, 0, 0, 0, IncludeAt

#define kNoResource 	-1
#define kSBarSizeMinus1 15

/*-------------------------------------view-------------------------------------*/
#define TCONTROLDATA	\
	/* Adornment flags	*/		unsigned binary byte;							\
	/* Pen size 		*/		align word; 									\
								point;											\
								boolean 	notSizeable, sizeable;				\
								boolean 	notDimmed, dimmed;					\
								boolean 	notHilited, hilited;				\
								boolean 	doesntDismiss, dismisses;			\
								align word; 									\
																				\
	/* Control inset	*/		rect;											\
																				\
	/* Font Style		*/		unsigned byte	plain;							\
								align word; 									\
	/* Font Size		*/		integer;										\
	/* Font RGBColor	*/		wide array [3] { unsigned hex integer; };		\
	/* Font name		*/		pstring;										\
								align word

#define TBUTTONDATA 	\
	/* Button label 	*/		pstring;										\
								align word

#define TCHECKBOXDATA	\
	/* Initially on 	*/		byte		off, on;							\
								align word; 									\
	/* Check box label	*/		pstring;										\
								align word

#define TRADIODATA		\
	/* Initially on 	*/		byte		off, on;							\
								align word; 									\
	/* Radio label		*/		pstring;										\
								align word

#define TSCROLLBARDATA	\
	/* Initial value	*/		longint;										\
	/* Minimum value	*/		longint;										\
	/* Maximum value	*/		longint

#define TCLUSTERDATA	\
	/* Cluster label	*/		pstring;										\
								align word

#define TICONDATA		\
	/* If 'cicn' first	*/		byte		dontPreferColor, preferColor;		\
								align word; 									\
	/* ICON or cicn id	*/		integer

#define TPATTERNDATA	\
	/* If 'ppat' first	*/		byte		dontPreferColor, preferColor;		\
								align word; 									\
	/* PAT	or ppat id	*/		integer

#define TPICTUREDATA	\
	/* PICT rsrc id 	*/		integer

#define TPOPUPDATA		\
	/* MENU rsrc ID 	*/		integer;										\
	/* Current item #	*/		integer;										\
	/* Item left offset */		integer

#define TSTATICTEXTDATA \
	/* Justification	*/		integer 	forceLeft = -2, justRight,			\
											justSystem, justCenter;				\
	/* Initial string	*/		pstring;										\
								align word

#define justLeft 		justSystem
	/* see comments in UMacAppUtilities about justLeft (left in for compatibility (2.0)	*/

#define TEDITTEXTDATA	\
	/* Max. characters	*/		integer 	unlimited = 255;					\
	/* Good cntl chars	*/		unsigned binary longint

#define TNUMBERTEXTDATA \
	/* Initial value	*/		longint;										\
	/* Minimum value	*/		longint;										\
	/* Maximum value	*/		longint

#define TTEVIEWDATA 	\
	/* Styled flag		*/		boolean 	withoutStyle, withStyle;			\
	/* Auto wrapping	*/		boolean 	crOnly, autoWrap;					\
	/* Changes flag 	*/		boolean 	dontAcceptChanges, acceptChanges;	\
	/* Free text flag	*/		boolean 	dontFreeText, freeText; 			\
								align word; 									\
	/* Key cmd number	*/		integer 	cTyping = 120;						\
	/* Max number chars */		integer 	unlimited = 32767;					\
	/* Inset			*/		rect;											\
	/* Justification	*/		integer 	forceLeft = -2, justRight,			\
											justSystem, justCenter;				\
	/* Font Style		*/		unsigned byte	plain;							\
								align word; 									\
	/* Font Size		*/		integer;										\
	/* Font RGBColor	*/		wide array [3] { unsigned hex integer; };		\
	/* Font name		*/		pstring;										\
								align word

#define TGRIDVIEWDATA	\
	/* Number of rows	*/		integer;										\
	/* Number of cols	*/		integer;										\
	/* row height		*/		integer;										\
	/* column width 	*/		integer;										\
	/* row inset		*/		integer;										\
	/* column inset 	*/		integer;										\
	/* adorn rows		*/		boolean 	dontAdornRows, AdornRows;			\
	/* adorn columns	*/		boolean 	dontAdornCols, AdornCols;			\
	/* single select.	*/		boolean 	multipleSelection, singleSelection; \
								align		word

#define TTEXTGRIDVIEWDATA	\
	/* Font Style		*/		unsigned byte plain;							\
								align word; 									\
	/* Font Size		*/		integer;										\
	/* Font RGBColor	*/		wide array [3] { unsigned hex integer; };		\
	/* Font name		*/		pstring;										\
								align word
								
/*	TTEXTLISTVIEWDATA is NULL */


type 'view' {
	integer = $$CountOf(viewArray); 	// Number of views we're defining
	wide array viewArray {

		/* TView data */
		literal longint root = '    ';	// Signature of ancestral view
		literal longint noID = '    ';	// Identifier for this view, if any

		wide array [1] { longint; longint; };	// Location inside parent view
		wide array [1] { longint; longint; };	// Size of view

		unsigned bitstring[3]			// Vertical view determiner
					sizeSuperView,		// View is the same size as superview
					sizeRelSuperView,	// Size is relative to the superview
					sizePage,			// View to be the size of one page
					sizeFillPages,		// View fills an exact number of pages
					sizeVariable,		// Size criteria is app-specific
					sizeFixed;			// No special default handling of size

		unsigned bitstring[3]			// Horizontal view determiner
					sizeSuperView,		// View is the same size as superview
					sizeRelSuperView,	// Size is relative to the superview
					sizePage,			// View to be the size of one page
					sizeFillPages,		// View fills an exact number of pages
					sizeVariable,		// Size criteria is app-specific
					sizeFixed;			// No special default handling of size

		boolean 	notShown, shown;
		boolean 	disabled, enabled;
		align word;

		switch {

		case Include:
			key literal longint = 'incl';				// Template signature
			integer;									// id of included view rsrc
			align		word;

		case IncludeAt:
			key literal longint = 'inc@';				// Template signature
			integer;									// id of included view rsrc
			wide array [1] { longint; longint; };		// Location to offset subviews			
			align		word;

		case View:
			key literal longint = 'view';				// Template signature
			pstring;									// Class name (e.g. "TView")
			align word;

		case DefaultView:
			key literal longint = 'dflt';				// Template signature
			pstring;									// Class name (e.g. "TDefaultView")
			align word;

		case Window:
			key literal longint = 'wind';				// Template signature
			pstring;									// Class name (eg "TWindow")
			align		word;
			integer 	documentProc,					// Window procID
						dBoxProc,
						plainDBox,
						altDBoxProc,
						noGrowDocProc,
						zoomDocProc = 8,
						zoomNoGrow = 12,
						rDocProc = 16;
			boolean 	noGoAwayBox, goAwayBox;
			boolean 	notResizable, resizable;
			boolean 	modeless, modal;
			boolean 	ignoreFirstClick, doFirstClick;
			boolean 	dontFreeOnClosing, freeOnClosing;
			boolean 	dontDisposeOnFree, disposeOnFree;
			boolean 	doesntCloseDocument, closesDocument;
			boolean 	dontOpenWithDocument, openWithDocument;
			boolean 	dontAdaptToScreen, adaptToScreen;
			boolean 	dontStagger, stagger;
			boolean 	dontForceOnScreen, forceOnScreen;
			unsigned bitstring[2]	dontCenter, centerHorizontally,
									centerVertically, center;
			align		word;
			literal longint noID = '    ';				// ID of the target view
			pstring;									// Window title
			align		word;

		case Scroller:
			key literal longint = 'scrl';				// Template signature
			pstring;									// Class name, "TScroller"
			align word;
			byte		noVertScrollBar, VertScrollBar;
			byte		noHorzScrollBar, HorzScrollBar;
			longint;									// Vertical maximum.
			longint;									// Horizontal maximum.
			integer;									// Vertical scrollunits.
			integer;									// Horizontal scroll units.
			byte		noVertConstrain, VertConstrain;
			byte		noHorzConstrain, HorzConstrain;
			rect;										// Scroll bar offsets

		case DialogView:
			key literal longint = 'dlog';				// Template signature
			pstring;									// Class name
			align word;
			literal longint noID = '    ';				// Id of default item
			literal longint noID = '    ';				// Id of cancel item

		case Control:
			key literal longint = 'cntl';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;

		case Button:
			key literal longint = 'butn';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TBUTTONDATA;

		case CheckBox:
			key literal longint = 'chkb';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TCHECKBOXDATA;

		case Radio:
			key literal longint = 'radb';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TRADIODATA;

		case ScrollBar:
			key literal longint = 'sbar';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TSCROLLBARDATA;

		case Cluster:
			key literal longint = 'clus';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TCLUSTERDATA;

		case Icon:
			key literal longint = 'icon';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TICONDATA;

		case Pattern:
			key literal longint = 'patn';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TPATTERNDATA;

		case Picture:
			key literal longint = 'pict';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TPICTUREDATA;

		case Popup:
			key literal longint = 'popp';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TPOPUPDATA;

		case StaticText:
			key literal longint = 'stat';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TSTATICTEXTDATA;

		case EditText:
			key literal longint = 'edit';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TSTATICTEXTDATA;
			TEDITTEXTDATA;

		case NumberText:
			key literal longint = 'nmbr';				// Template signature
			pstring;									// Class name
			align word;

			TCONTROLDATA;
			TSTATICTEXTDATA;
			TEDITTEXTDATA;
			TNUMBERTEXTDATA;

		case TEView:
			key literal longint = 'tevw';				// Template signature
			pstring;									// Class name
			align word;

			TTEVIEWDATA;

		case GridView:
			key literal longint = 'grid';				// Template signature
			pstring;									// Class name
			align		word;

			TGRIDVIEWDATA;

		case TextGridView:
			key literal longint = 'txtg';				// Template signature
			pstring;									// Class name
			align		word;

			TGRIDVIEWDATA;
			TTEXTGRIDVIEWDATA;

		case TextListView:
			key literal longint = 'lstg';				// Template signature
			pstring;									// Class name
			align		word;
			
			TGRIDVIEWDATA;
			TTEXTGRIDVIEWDATA;

		case OTHER:
			key literal longint = 'othr';				// Template signature
			pstring;									// Class name
			align word;
			integer = (($$CountOf(dataArray) + 5) / 2 ) * 2;
			literal longint;							// Actual signature
			wide array dataArray {
				unsigned hex byte;
			};
			align word;
#IFDEF XIncludeOtherViewTypes			// auto-defined by script system
#IF XIncludeOtherViewTypes
#Include $$Shell("OtherViewTypesSrc");	// let end users extend the view type
#ENDIF
#ENDIF
		};
	};
};
#endif
